!!ARBfp1.0
# fragment shader for 3D turbulence-based wood

ATTRIB frame = fragment.texcoord[0];
ATTRIB noiseInput1 = fragment.texcoord[1]; # xy, 1st octave
ATTRIB noiseInput2 = fragment.texcoord[2]; # xy, 2nd octave
ATTRIB noiseInput4 = fragment.texcoord[3]; # xy, 3rd octave
ATTRIB noiseInput8 = fragment.texcoord[4]; # xy, 4th octave
ATTRIB noiseInputZ = fragment.texcoord[5]; # z, all 4 octaves
ATTRIB woodSpace   = fragment.texcoord[6]; # 3D input
TEMP intArg, fracArg, fade, grad;
TEMP hash, tex, noise;
TEMP t0,t1, n0,n1, turb, turb1;

# base for blum blum shub should be product of two primes
# but I couldn't find any product that looked good and didn't
# result in huge texture sizes. 61 is a single prime that seemed 
# to produce reasonably good results
PARAM base = {61., 0.0163934}; # base, 1/base

# running short on constants, so I stuffed some here
PARAM c = {0,1,2,3};

# integer and fractional components of input, and fade func
FRC fracArg,noiseInputZ;
FLR intArg,noiseInputZ;
MAD fade,fracArg,-c.z,c.w;# old fade function t*t*(-2*t+3)
MUL fade,fade,fracArg;		# common part
MUL fade,fade,fracArg;

# for 1x and 2x frequency:
# compute z & z+1 terms and run through hash function
MAD hash,intArg.xxyy,3,{0,3,0,3};
MUL hash,hash,hash;
MUL hash,hash,base.y;
FRC hash,hash;

# 1x frequency noise
MOV t0.x,noiseInput1.x;        MOV t1.x,noiseInput1.x;
ADD t0.y,noiseInput1.y,hash.x; ADD t1.y,noiseInput1.y,hash.y;
TEX n0.xy,t0,texture[0],2D;
TEX n1.xy,t1,texture[0],2D;
MAD noise.xy,n0,c.z,-c.y;
MAD noise.zw,n1.xyxy,c.z,-c.y;
ADD grad,fracArg.x,-c.xxyy;
MUL noise.yw,noise,grad;
ADD turb.x,noise.z,noise.w;
ADD turb1.x,noise.x,noise.y;

# 2x frequency noise
MOV t0.x,noiseInput2.x;        MOV t1.x,noiseInput2.x;
ADD t0.y,noiseInput2.y,hash.z; ADD t1.y,noiseInput2.y,hash.w;
TEX n0.xy,t0,texture[0],2D;
TEX n1.xy,t1,texture[0],2D;
MAD noise.xy,n0,c.z,-c.y;
MAD noise.zw,n1.xyxy,c.z,-c.y;
ADD grad,fracArg.y,-c.xxyy;
MUL noise.yw,noise,grad;
ADD turb.y,noise.z,noise.w;
ADD turb1.y,noise.x,noise.y;

# for 4x and 8x frequency:
# compute z & z+1 terms and run through hash function
MAD hash,intArg.zzww,3,{0,3,0,3};
MUL hash,hash,hash;
MUL hash,hash,base.y;
FRC hash,hash;

# 4x frequency noise
MOV t0.x,noiseInput4.x;        MOV t1.x,noiseInput4.x;
ADD t0.y,noiseInput4.y,hash.x; ADD t1.y,noiseInput4.y,hash.y;
TEX n0.xy,t0,texture[0],2D;
TEX n1.xy,t1,texture[0],2D;
MAD noise.xy,n0,c.z,-c.y;
MAD noise.zw,n1.xyxy,c.z,-c.y;
ADD grad,fracArg.z,-c.xxyy;
MUL noise.yw,noise,grad;
ADD turb.z,noise.z,noise.w;
ADD turb1.z,noise.x,noise.y;

# 8x frequency noise
MOV t0.x,noiseInput8.x;        MOV t1.x,noiseInput8.x;
ADD t0.y,noiseInput8.y,hash.z; ADD t1.y,noiseInput8.y,hash.w;
TEX n0.xy,t0,texture[0],2D;
TEX n1.xy,t1,texture[0],2D;
MAD noise.xy,n0,c.z,-c.y;
MAD noise.zw,n1.xyxy,c.z,-c.y;
ADD grad,fracArg.w,-c.xxyy;
MUL noise.yw,noise,grad;
ADD turb.w,noise.z,noise.w;
ADD turb1.w,noise.x,noise.y;

# combine with fade functions & sum octaves
LRP turb,fade,turb,turb1;
ABS turb,turb;
DP4 turb,turb,{1,.5,.25,.125};

# compute wood ring distance
TEMP p, d, ring, wcol;
MAD p,turb,frame.r,woodSpace;
PARAM a = {0.673, 0.74, 0, 0};
DP3 d.x,p,a;	  # p.a
MAD d,-d.x,a,p;	  # p-p.a = vector from tree axis
DP3 d.x,d,d;  # squared distance from tree axis
MOV wcol,{.49,.42,.15};
MUL d.x,d.x,frame.g;
FRC d.x,d.x;
MAD wcol,wcol,d.x,wcol;

# combine with diffuse lighting
MUL result.color,wcol,fragment.color;
END